package rtmail ; 

require Exporter;   
@ISA = ('Exporter');

@EXPORT=qw(rtmail maildomain) ;

use Mail::Sender ;
use MIME::Base64 ; 
use strict ; 

use perlpf ;

our $Debug = 0 ; 

sub fixlist { 
    my ($s) = @_ ;
    if ( $s =~ / / ) { 
	$s =~ s/^\s+// ; 
	$s =~ s/s+$// ; 
	my @list = split(/[\s,]/, $s ) ; 
	my %m ;
	foreach (@list) { 
	    $m{$_} = 1 if $_ ne "" ; 
	}
	$s = join(',', sort keys %m) ; 
    }
    return $s ; 
}


sub rtmail { 
    my $retcode = 0 ; 
    my %hash = @_ ;
    my $to = &fixlist($hash{-to}) ; 
    my $from = $hash{-from} ; 
    my $subject = $hash{-subject} ; 
    my $cc = &fixlist($hash{-cc}) ; 
    my $bcc = &fixlist($hash{-bcc}) ; 
    my $attach = &fixlist($hash{-attach}) ; 
    my $msg = $hash{-msg} ; 
    my $errmsg ;
    $Debug = $hash{-debug} ;

    if ( $hash{-background} ) { 
	my $pid ; 
	if ( $pid = fork ) {
	    # parent
	    return ; 
	} elsif ( defined $pid )   {
	    # child
	    close STDIN ; 
	    if ( $] > 5 ) {
		eval "setpgrp 0" ;
	    } else {
		setpgrp (0, $$) ;
	    }
	} else {
	    die "Can't fork\n" ;
	}
    }

    if ( ! defined $from || $from =~ /^\s*$/) { 
	$from = getpwuid($<);
    }

    my ($host, $domain) = &maildomain() ;
    my $result = -1 ;
    my @tried = () ;

    my $mailfrom ;
    if ( $from !~ /\@/ ) { 
	$mailfrom = "$from\@$host" ; # use fully qualified name for local machine
    } else { 
	$mailfrom = $from ; 
    }
    my $mailhost = pfget("site.pf", "mailhost") ; 
    my @mailers = ($host, "localhost") ;
    unshift(@mailers, $mailhost) if defined $mailhost ;
    foreach my $mailer (@mailers) { 
	my ($mailhost) = gethostbyname($mailer) ; 
	if (defined $mailhost ) { 
	    ($result, $errmsg) = &send_the_mail( $mailfrom, $subject, $to, $cc, $bcc, $mailhost, $msg, $attach) ; 
	    push(@tried, $mailer, $errmsg) ;
	    if ( $result == 0 ) { 
		$msg = "sent mail using $mailhost to $to from $mailfrom\n" ; 
		last ;
	    }
	}
    }

# mail.brtt.com and its ip address are last gasp efforts for license requests and incident reports
    if ( $result != 0 ) { 
	if ( $from !~ /\@/ ) { 
	    $mailfrom = "$from\@$domain" ; 
	} else { 
	    $mailfrom = $from ; 
	}
	my @mailers = ($mailhost, "mailhost",  "mail", "mail.brtt.com", "207.174.76.131" ) ;
	foreach my $mailer (@mailers) { 
	    my ($mailhost) = gethostbyname($mailer) ; 
	    if (defined $mailhost ) { 
		($result, $errmsg) = &send_the_mail( $mailfrom, $subject, $to, $cc, $bcc, $mailhost, $msg, $attach) ; 
		push(@tried, $mailer, $errmsg) ;
		if ( $result == 0 ) { 
		    $msg = "sent mail using $mailhost to $to from $mailfrom\n" ; 
		    last ;
		}
	    }
	}
    }

# just in case the mail program is actually working.
    if ( $result != 0 ) { 
	$result = &send_by_mail ( $subject, $to, $cc, $bcc, $msg, $attach ) ; 
	if ( $result == 0 ) { 
	    $msg = "sent mail using mail program to $to\n" ; 
	}
    }

    if ( $result != 0 ) { 
	$msg = "failed to send mail from $from using any of these servers: @tried\n\tand the mail program\n" ;
    }

    return ($result, $msg) ;
}

sub send_by_mail {
    my ( $subject, $to, $cc, $bcc, $msg, $attach ) = @_ ;  
    my $result ;
    my $cmd = "mail" ; 
    $cmd .= " -c $cc" if $cc ne "" ; 
    $cmd .= " -b $bcc" if $bcc ne "" ; 
    $cmd .= " -s '$subject'" if $subject ne "" ; 
    $to =~ s/,/ /g ; 
    $cmd .= " $to" ;
    my $pid = open ( MAIL, "| $cmd > /dev/null 2>&1" ) ; 
    if ( defined $pid ) { 
	if ( defined $attach && $attach ne "" ) { 
	    $result = send_attachments($msg, $attach ) ;
	} else { 
	    print MAIL $msg ; 
	}
	close MAIL ; 
	$result = $? ;
	warn ( "rtmail to $to using regular mail program failed:\n" ) if $Debug && $result != 0 ;
    } else { 
	$result = -1 ; 
    }
    return $result ;
}

sub send_attachments { 
    my ($msg, $attach ) = @_ ; 
    my @attachments = split(',', $attach ) ; 
    my $boundary = "antelope__$$" ;

print MAIL <<"EOF1" ;
MIME-Version: 1.0
Content-Type: MULTIPART/mixed; BOUNDARY=$boundary

EOF1

if ( $msg ne "" ) { 
    print MAIL <<"EOFmsg";
--$boundary
Content-Type: TEXT/plain; charset=us-ascii

EOFmsg

print MAIL $msg ; 
}

foreach my $filename (@attachments) { 
    my $name = $filename ; 
    $name =~ s".*/"" ;
    my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
	 $atime,$mtime,$ctime,$blksize,$blocks)
				 = stat($filename);
    my $mode = sprintf ( "0%04o", $mode ) ;

print MAIL <<"EOF2" ;

--$boundary
Content-Type: APPLICATION/octet-stream; name="$name"; x-unix-mode=$mode
Content-Description: $name
Content-Transfer-Encoding: BASE64

EOF2

open ( FILE, $filename ) ; 
my @file = <FILE> ; 
close FILE ; 
my $file = join ('', @file ) ; 
$file = encode_base64($file) ;
print MAIL $file ; 
}

print MAIL <<"EOF3" ;

--$boundary--

EOF3

}

sub maildomain {
    chomp ( my $host = `uname -n` ) ;
    chomp ( my $os = `uname -s` ) ;
    my $domain ;
    $domain = pfget("site.pf", "mail_domain" ) ; 
    if ( defined $domain && $domain =~ m"[ !\"#$%&'()*+,/:;<->?@[\\\]^_`{|}~]" ) { 
	warn ( "rtmail::maildomain: Discarding unlikely domain name: '$domain'\n Did you edit $ENV{ANTELOPE}/site.pf?" ) ; 
	$domain = undef ; 
    }
    if ( ! defined $domain || $domain eq "" ) { 
	if ( $os eq "Linux" ) {
	    open ( DOMAIN, "hostname --domain 2>/dev/null |" ) ; 
	    my @domain = <DOMAIN> ; 
	    close DOMAIN ;
	    if ( @domain == 1 ) { 
		$domain = $domain[0] ; 
		chomp($domain) ;
	    }
	}
	if ( ! defined $domain || $domain eq "" ) {
	    open ( RESOLV, "/etc/resolv.conf" ) ;
	    while ( <RESOLV> ) {
		if ( /^\s*domain\s+(\S+)/ ) {
		    $domain = $1 ;
		    last ;
		}
	    }
	    close RESOLV ;
	}
	if ( ! defined $domain || $domain eq "" ) { 
	    if ( $host =~ /\w+\.(\w+\.\w+)/ ) {
		$domain = $1 ; 
	    } else { 
		$domain = "brtt.net" ; 
	    }
	}
    }

    if ( $host !~ /\w+\.\w+\.\w+/ ) {
        $host = "$host.$domain" ;
    }
    return ($host, $domain) ;
}

sub send_the_mail {
    my ($from, $subject, $to, $cc, $bcc, $mailhost, $msg, $attach) = @_ ;

    my $sender = new Mail::Sender
	{smtp => $mailhost, from => "$from"} ;
    $msg = " " if $msg eq "" ;
    my $result ;
    my $retcode = -1 ;
    if ( defined $attach && length($attach) > 0) { 
	$result = $sender->MailFile({
	    to => $to,
	    cc => $cc,
	    bcc => $bcc,
	    subject => $subject,
	    msg => $msg,
	    file => $attach,
	    }) ;
    } else { 
	$result = $sender->MailMsg({
	    to => $to,
	    cc => $cc, 
	    bcc => $bcc,
	    subject => $subject,
	    msg => $msg,
	    }) ;
    }
    my $errmsg = "" ; 
    if ( ! ref($result) && ( $result == -2 || $result == -3 || $result == -5 || $result == -7 ))  { 
	$errmsg = my_error($mailhost, $to, $subject) ; 
    } else { 
	if ( ! ref($result) ) { 
	    $errmsg = my_error($mailhost, $to, $subject) ; 
	} else {
	    $retcode = 0 ; 
	}
    }
    $sender->Close ;
    return ($retcode, $errmsg) ; 
}

sub my_error {
    my ($mailhost, $to, $subject) = @_ ;
    return "rtmail to '$to'\@'$mailhost' Subject:'$subject' failed: $Mail::Sender::Error\n" ;
}

1;
